# coding=utf-8
# Copyright (c) 2015 EMC Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import unicode_literals

import logging

from storops.exception import UnityQuotaConfigModifyException, \
    UnityResourceNotFoundError
from storops.unity.resource import UnityResource, UnityResourceList

__author__ = 'Rajendra Indukuri'

log = logging.getLogger(__name__)


class UnityQuotaConfig(UnityResource):
    """
    Support for Unity quotaConfig component

    Operations supported:
    modify: Modify quota_config using the quota_config_id
    """

    @classmethod
    def modify(cls, cli, quota_config_id,
               quota_policy=None,
               is_user_quota_enabled=None,
               delete_user_quotas_with_disable=None,
               is_access_deny_enabled=None,
               grace_period=None,
               default_hard_limit=None,
               default_soft_limit=None):
        """
        Modifies tree_quota params for the specified tree_quota_id
        :param quota_config_id: This is required which specifies
            quota_config to be modified
        :param quota_policy: This is an enum which specifies how
            disk usage shold be measured in blocks/file_size
        :param is_user_quota_enabled: To see if user_quota is
            enabled. Cannot be passed with quota_policy
        :param delete_user_quotas_with_disable: whether to delete
            user_quotas when disabling user quotas
        :param is_access_deny_enabled: when true access will be
            denied when limit is exceeded
        :param grace_period: Grace period for soft limit
        :param default_hard_limit: Default hard limit of user quotas
            and tree quotas
        :param default_soft_limit: Default soft limit of user quotas
            and tree quotas.
        :return: None.
        """
        quota_config = UnityQuotaConfig.get(_id=quota_config_id, cli=cli)
        if not quota_config.existed:
            raise UnityResourceNotFoundError(
                     'cannot find quota_config {}.'.format(quota_config_id))

        # quota_policy and is_user_quota_enabled cannot be used together
        if quota_policy is not None and is_user_quota_enabled is not None:
            raise UnityQuotaConfigModifyException()

        req_body = cli.make_body(
            quotaPolicy=quota_policy,
            isUserQuotaEnabled=is_user_quota_enabled,
            deleteUserQuotasWithDisable=delete_user_quotas_with_disable,
            isAccessDenyEnabled=is_access_deny_enabled,
            gracePeriod=grace_period,
            defaultHardLimit=default_hard_limit,
            defaultSoftLimit=default_soft_limit
        )
        resp = cli.action(cls().resource_class, quota_config_id,
                          'modify', **req_body)
        resp.raise_if_err()
        return resp


class UnityQuotaConfigList(UnityResourceList):
    """
    List representation of quota_config
    """

    def __init__(self, cli=None, **filters):
        super(UnityQuotaConfigList, self).__init__(cli, **filters)

    @classmethod
    def get_resource_class(cls):
        return UnityQuotaConfig



# coding=utf-8
# Copyright (c) 2015 EMC Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import unicode_literals

import logging

import storops
from storops.exception import UnityResourceNotFoundError
from storops.unity.resource import UnityResource, UnityResourceList
from storops.unity.client import UnityClient

__author__ = 'Rajendra Indukuri'

log = logging.getLogger(__name__)


class UnityTreeQuota(UnityResource):
    """
    Support CRUD operations for tree_quota

    Operations supported:
    create: Create tree_quota on the file_system path specified
    modify: Modify tree_quota on the file_system path specified
    delete: Delete tree_quota on the file_system using id
    """

    @classmethod
    def create(cls, cli, filesystem_id=None, path=None,
               description=None, hard_limit=None,
               soft_limit=None):
        """
        Creates tree_quota on the specified filesystem and path
        :param filesystem_id: This parameter will help create
            tree_quota on the filesystem and folder path specified
        :param path: Path of the tree quota relative to the root of
            the file system
        :param description: Description of the tree quota
        :param hard_limit: sets hard_limit on the file_system path
            specified. The default value 0 means no limitation
        :param soft_limit: sets soft_limit on the file_system path
            specified. The default value 0 means no limitation
        :return: instance of tree quota created.
        """
        fs_clz = storops.unity.resource.filesystem.UnityFileSystem
        filesystem = fs_clz.get(cli, filesystem_id)
        if not filesystem.existed:
            raise UnityResourceNotFoundError(
                'cannot find filesystem {}.'.format(filesystem_id))

        tree_quota_param = cls.prepare_tree_quota_create_parameters(
            filesystem, path, description, hard_limit, soft_limit)
        resp = cli.post(
            cls().resource_class, **tree_quota_param)
        resp.raise_if_err()
        return cls(_id=resp.resource_id, cli=cli)

    @classmethod
    def modify(cls, cli, tree_quota_id, description=None,
               hard_limit=None, soft_limit=None):
        """
        Modifies tree_quota params for the specified tree_quota_id
        :param tree_quota_id: This is required which specifies tree_quota to
            be modified
        :param description: Modified description of tree_quota
        :param hard_limit: modifies hard_limit for the
            tree_quota specified
        :param soft_limit: modifies soft_limit for the
            tree_quota specified
        :return: None
        """
        tree_quota = UnityTreeQuota.get(_id=tree_quota_id, cli=cli)
        if not tree_quota.existed:
            raise UnityResourceNotFoundError(
                'cannot find tree_quota {}.'.format(tree_quota_id))

        req_body = cli.make_body(
            description=description,
            hardLimit=hard_limit,
            softLimit=soft_limit
        )
        resp = cli.action(cls().resource_class, tree_quota_id,
                          'modify', **req_body)
        resp.raise_if_err()
        return resp

    @classmethod
    def delete(cls, cli, tree_quota_id):
        """
        Delete tree_quota
        :param tree_quota_id: Quota id of the tree_quota to be deleted
        :return: None
        """
        tree_quota = UnityTreeQuota.get(_id=tree_quota_id, cli=cli)
        if not tree_quota.existed:
            raise UnityResourceNotFoundError(
                'cannot find tree_quota {}.'.format(tree_quota_id))

        resp = cli.delete(cls().resource_class,
                          tree_quota_id)
        resp.raise_if_err()
        return resp

    @staticmethod
    def prepare_tree_quota_create_parameters(
            filesystem, path, description, hard_limit, soft_limit):
        """
        Prepare tree_quota for the create operation
        :param filesystem: Filesystem on which tree_quota needs to
            be created
        :param path: Path of the tree quota relative to the root of
            the file system
        :param description: Description of the tree_quota
        :param hard_limit: hard_limit for the tree_quota
        :param soft_limit: soft_limit for the tree_quota
        :return: tree_quota_params for create tree_quota preparation
        """
        tree_quota_param = UnityClient.make_body(
            filesystem=filesystem,
            path=path,
            description=description,
            hardLimit=hard_limit,
            softLimit=soft_limit)
        return tree_quota_param


class UnityTreeQuotaList(UnityResourceList):
    """
    List representation of tree_quota
    """

    def __init__(self, cli=None, **filters):
        super(UnityTreeQuotaList, self).__init__(cli, **filters)

    @classmethod
    def get_resource_class(cls):
        return UnityTreeQuota



# coding=utf-8
# Copyright (c) 2015 EMC Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import unicode_literals

from storops.unity.resource import UnityResource, UnityResourceList

__author__ = 'Cedric Zhuang'


class UnityDataStore(UnityResource):
    pass


class UnityDataStoreList(UnityResourceList):
    @classmethod
    def get_resource_class(cls):
        return UnityDataStore


class UnityVmDisk(UnityResource):
    pass


class UnityVmDiskList(UnityResourceList):
    @classmethod
    def get_resource_class(cls):
        return UnityVmDisk


class UnityVm(UnityResource):
    pass


class UnityVmList(UnityResourceList):
    @classmethod
    def get_resource_class(cls):
        return UnityVm


class UnityVirtualVolume(UnityResource):
    pass


class UnityVirtualVolumeList(UnityResourceList):
    @classmethod
    def get_resource_class(cls):
        return UnityVirtualVolume


class UnityHostVvolDatastore(UnityResource):
    pass


class UnityHostVvolDatastoreList(UnityResourceList):
    @classmethod
    def get_resource_class(cls):
        return UnityHostVvolDatastore


class UnityCapabilityProfile(UnityResource):
    pass


class UnityCapabilityProfileList(UnityResourceList):
    @classmethod
    def get_resource_class(cls):
        return UnityCapabilityProfile


class UnityVirtualVolumeBinding(UnityResource):
    pass


class UnityVirtualVolumeBindingList(UnityResourceList):
    @classmethod
    def get_resource_class(cls):
        return UnityVirtualVolume


class UnityVmwarePE(UnityResource):
    pass


class UnityVmwarePEList(UnityResourceList):
    @classmethod
    def get_resource_class(cls):
        return UnityVmwarePE


class UnityVmwareNasPEServer(UnityResource):
    pass


class UnityVmwareNasPEServerList(UnityResourceList):
    @classmethod
    def get_resource_class(cls):
        return UnityVmwareNasPEServer



# coding=utf-8
# Copyright (c) 2017 Dell Inc. or its subsidiaries.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import operator


class UnityModel:
    def __init__(self, model):
        self.model = model.split(' ')[-1]
        self.series = self.model[0]

        if self.model.endswith('F'):
            self.is_all_flash = True
            self.model_base = self.model.rstrip('F')
        else:
            self.is_all_flash = False
            self.model_base = self.model

    def is_same_series(self, other):
        return (self.series == other.series) and (
                self.is_all_flash == other.is_all_flash)

    def compare(self, other, op):
        if not self.is_same_series(other):
            return False
        return op(int(self.model_base), int(other.model_base))

    def __eq__(self, other):
        return self.compare(other, operator.eq)

    def __gt__(self, other):
        return self.compare(other, operator.gt)

    def __ge__(self, other):
        return self.compare(other, operator.ge)

    def __lt__(self, other):
        return self.compare(other, operator.lt)

    def __le__(self, other):
        return self.compare(other, operator.le)



# coding=utf-8
# Copyright (c) 2020 Dell Inc. or its subsidiaries.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from __future__ import unicode_literals

import tempfile
from unittest import TestCase

from hamcrest import equal_to, assert_that

from storops import SGCache


class SGCacheTest(TestCase):
    def test_set_and_get(self):
        folder = tempfile.mkdtemp(suffix='storops')
        cache = SGCache(folder)
        cache['sg-1'] = {'abc': 'efg'}

        cache_get = SGCache(folder)
        assert_that('sg-1' in cache_get, equal_to(True))
        assert_that('abc' in cache_get['sg-1'], equal_to(True))
        assert_that(cache_get['sg-1']['abc'], equal_to('efg'))



# Copyright (c) 2015 EMC Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import unicode_literals

from unittest import TestCase

import ddt

from hamcrest import assert_that, equal_to, instance_of, raises

from storops.exception import UnityQuotaConfigModifyException, \
    UnityResourceNotFoundError
from storops.unity.enums import QuotaPolicyEnum
from storops.unity.resource.quota_config import UnityQuotaConfigList, \
     UnityQuotaConfig
from storops_test.unity.rest_mock import t_rest, patch_rest

__author__ = 'Rajendra Indukuri'


@ddt.ddt
class UnityQuotaConfigListTest(TestCase):
    @patch_rest
    def test_get_all(self):
        quotas_configs = UnityQuotaConfigList(cli=t_rest())
        assert_that(quotas_configs, instance_of(UnityQuotaConfigList))
        assert_that(len(quotas_configs), equal_to(2))

    @patch_rest
    def test_get_specific_quota_config(self):
        quota_config = UnityQuotaConfig(
                          _id='quotaconfig_171798691845_1', cli=t_rest())
        assert_that(quota_config, instance_of(UnityQuotaConfig))
        assert_that(quota_config.id,
                    equal_to('quotaconfig_171798691845_1'))
        assert_that(quota_config.existed, equal_to(True))

    @patch_rest
    def test_get_quota_config_not_found(self):
        quota_config = UnityQuotaConfig(_id='abc', cli=t_rest())
        assert_that(quota_config.existed, equal_to(False))

    @patch_rest
    def test_modify_quota_config(self):
        resp = UnityQuotaConfig.modify(
                cli=t_rest(), quota_config_id='quotaconfig_171798691845_1',
                quota_policy=QuotaPolicyEnum.BLOCKS,
                is_user_quota_enabled=None,
                delete_user_quotas_with_disable=False,
                is_access_deny_enabled=False,
                grace_period=345600,
                default_hard_limit=8589934592,
                default_soft_limit=2147483648)
        assert_that(resp.is_ok(), equal_to(True))

    @patch_rest
    def test_modify_quota_config_invalid_quota_config_id(self):
        def f():
            UnityQuotaConfig.modify(cli=t_rest(),
                                    quota_config_id='abc',
                                    quota_policy=QuotaPolicyEnum.BLOCKS,
                                    is_user_quota_enabled=None,
                                    delete_user_quotas_with_disable=False,
                                    is_access_deny_enabled=False,
                                    grace_period=345600,
                                    default_hard_limit=8589934592,
                                    default_soft_limit=2147483648)

        assert_that(f, raises(UnityResourceNotFoundError))

    @patch_rest
    def test_modify_quota_config_pass_policy_is_quota_enabled_together(self):
        def f():
            UnityQuotaConfig.modify(
                cli=t_rest(), quota_config_id='quotaconfig_171798691845_1',
                quota_policy=QuotaPolicyEnum.BLOCKS,
                is_user_quota_enabled=True,
                delete_user_quotas_with_disable=False,
                is_access_deny_enabled=False,
                grace_period=345600,
                default_hard_limit=8589934592,
                default_soft_limit=2147483648)

        assert_that(f, raises(UnityQuotaConfigModifyException))



# coding=utf-8
# Copyright (c) 2015 EMC Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import unicode_literals

import logging

import storops
from storops.exception import UnityResourceNotFoundError
from storops.unity.resource import UnityResource, UnityResourceList
from storops.unity.client import UnityClient

__author__ = 'Rajendra Indukuri'

log = logging.getLogger(__name__)


class UnityUserQuota(UnityResource):
    """
    Support CRUD operations for user_quota

    Operations supported:
    create: Create user_quota on a file_system/tree_quota
    modify: Modify user_quota on a file_system/tree_quota
    """
    @classmethod
    def create(cls, cli, filesystem_id=None, tree_quota_id=None,
               hard_limit=None, soft_limit=None, uid=None,
               unix_name=None, win_name=None):
        """
        Creates user_quota on the speficied filesystem or tree quota
        :param filesystem_id: This is an optional parameter which if provided
            will create user_quota on the filesystem specified.
        :param tree_quota_id: This is an optional parameter which if provided
            will create user_quota on the tree_quota specified.
        :param hard_limit: sets hard_limit on the filesystem for the user
            specified
        :param soft_limit: sets soft_limit on the filesystem for the user
            specified
        :param uid: This is an optional parameter to specify user. Either of
            uid/unix_name/win_name should be provided
        :param unix_name: This is an optional parameter to specify user. Either
            of uid/unix_name/win_name should be provided
        :param win_name:This is an optional parameter to specify user. Either
            of uid/unix_name/win_name should be provided
        :return: created user quota.
        """
        tree_quota = None
        filesystem = None
        if filesystem_id is not None:
            fs_clz = storops.unity.resource.filesystem.UnityFileSystem
            filesystem = fs_clz.get(_id=filesystem_id, cli=cli)
            if not filesystem.existed:
                raise UnityResourceNotFoundError(
                    'cannot find filesystem {}.'.format(filesystem_id))

        if tree_quota_id is not None:
            tquota_clz = storops.unity.resource.tree_quota.UnityTreeQuota
            tree_quota = tquota_clz.get(_id=tree_quota_id, cli=cli)
            if not tree_quota.existed:
                raise UnityResourceNotFoundError(
                    'cannot find tree_quota {}.'.format(tree_quota_id))

        user_quota_param = cls.prepare_user_quota_create_parameters(
            filesystem, tree_quota, hard_limit, soft_limit, uid,
            unix_name, win_name)
        resp = cli.post(
            cls().resource_class, **user_quota_param)
        resp.raise_if_err()
        return cls(_id=resp.resource_id, cli=cli)

    @classmethod
    def modify(cls, cli, user_quota_id, hard_limit=None, soft_limit=None):
        """
        Modifies user_quota params for the specified user_quota_id
        :param user_quota_id: This is required which speicfies user_quota to
            be modified
        :param hard_limit: modifies hard_limit on the file_system for the
            user_quota specified
        :param soft_limit: modifies soft_limit on the file_system for the
            user_quota specified
        :return: None
        """
        user_quota = UnityUserQuota.get(_id=user_quota_id, cli=cli)
        if not user_quota.existed:
            raise UnityResourceNotFoundError(
                'cannot find user_quota {}.'.format(user_quota_id))

        req_body = cli.make_body(
            hardLimit=hard_limit,
            softLimit=soft_limit
           )
        resp = cli.action(cls().resource_class, user_quota_id,
                          'modify', **req_body)
        resp.raise_if_err()
        return resp

    @staticmethod
    def prepare_user_quota_create_parameters(
            filesystem=None, tree_quota=None, hard_limit=None,
            soft_limit=None, uid=None, unix_name=None,
            win_name=None):
        """
        Prepare user_quota for the create operation
        :param filesystem: This is needed if the user_quota is to be
            created on the filesystem
        :param tree_quota: This is needed if the user_quota is to be
            created on the tree_quota
        :param hard_limit: hard_limit for the user_quota
        :param soft_limit: soft_limit for the user_quota
        :param uid: uid of the user
        :param unix_name: unix_name of the user
        :param win_name: windows name of the user
        :return: user_quota_params for create user_quota preparation
        """
        user_quota_param = UnityClient.make_body(
            filesystem=filesystem,
            treeQuota=tree_quota,
            hardLimit=hard_limit,
            softLimit=soft_limit,
            uid=uid,
            unixName=unix_name,
            winName=win_name)
        return user_quota_param


class UnityUserQuotaList(UnityResourceList):
    """
    List representation of user_quota
    """
    def __init__(self, cli=None, **filters):
        super(UnityUserQuotaList, self).__init__(cli, **filters)

    @classmethod
    def get_resource_class(cls):
        return UnityUserQuota



# coding=utf-8
# Copyright (c) 2015 EMC Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from __future__ import unicode_literals

import copy
import json
import logging
import time

import requests
import six
from requests.exceptions import ConnectTimeout
from requests.exceptions import RequestException
from retryz import retry

from storops.connection import exceptions
from storops.exception import StoropsConnectTimeoutError

log = logging.getLogger(__name__)


def _on_error_callback(e):
    return isinstance(e, RequestException)


def _wait_callback(tried):
    return 2 ** (tried - 1)


class HTTPClient(object):
    def __init__(self, base_url, headers, insecure=False, auth=None,
                 timeout=None, retries=None, ca_cert_path=None,
                 cache_interval=0):
        self.base_url = base_url
        if retries is None:
            retries = 2
        self.retries = retries
        self.request_options = self._set_request_options(
            insecure, auth, timeout, ca_cert_path)
        self.headers = headers
        self.session = requests.session()
        self.cache_interval = cache_interval
        self.cache_dict = {}

    def __del__(self):
        self.session.close()

    @staticmethod
    def _set_request_options(insecure=None, auth=None, timeout=None,
                             ca_cert_path=None):
        options = {'verify': True}

        if ca_cert_path is not None:
            options['verify'] = ca_cert_path

        if insecure:
            options['verify'] = False

        if timeout:
            options['timeout'] = timeout

        options['auth'] = auth

        return options

    def request(self, full_url, method, **kwargs):
        headers = copy.deepcopy(self.headers)
        headers.update(kwargs.get('headers', {}))
        options = copy.deepcopy(self.request_options)
        content_type = headers.get('Content-Type', None)
        if kwargs.get('body', None):
            if content_type == 'application/json':
                options['data'] = json.dumps(kwargs['body'])
            else:
                options['data'] = kwargs['body']
        files = kwargs.get('files', None)
        files_opener = None
        if files:
            files_opener = {}
            headers.pop('Content-Type')
            for name, path in files.items():
                files_opener[name] = open(path, 'rb')
        self.log_request(full_url, method, options.get('data', None))
        start = time.time()
        resp = self.session.request(method, full_url, headers=headers,
                                    files=files_opener, **options)

        self.log_response(full_url, method, resp, start)

        body = None
        if resp.text:
            try:
                if content_type == 'application/json':
                    body = json.loads(resp.text)
                else:
                    body = resp.text

            except ValueError:
                pass

        # These errors should be raised directly, do NOT retry any more,
        # like 401 - unauthorized, 503 - service unavailable.
        if resp.status_code in [401, 503]:
            raise exceptions.from_response(resp, method, full_url)

        return resp, body

    def _cs_request(self, url, method, **kwargs):
        try:
            return self._cs_request_with_retries(
                self.base_url + url,
                method,
                **kwargs)
        except ConnectTimeout as ex:
            raise StoropsConnectTimeoutError(message=str(ex))

    def _get_limit(self):
        return self.retries

    @retry(wait=_wait_callback, limit=_get_limit,
           on_error=_on_error_callback)
    def _cs_request_with_retries(self, url, method, **kwargs):
        return self.request(url, method, **kwargs)

    def get(self, url, **kwargs):
        if self.cache_interval <= 0:
            result = self._cs_request(url, 'GET', **kwargs)
        else:
            current = time.time()
            if url in self.cache_dict.keys() and (
                    current - self.cache_dict.get(url)[
                    0] < self.cache_interval):
                result = self.cache_dict[url][1]
                log.debug('Read response from cache, URL: {}'.format(url))
            else:
                result = self._cs_request(url, 'GET', **kwargs)
                current = time.time()
                log.debug('Write response to cache, URL: {}'.format(url))
                self.cache_dict[url] = (current, result)
        return result

    def post(self, url, **kwargs):
        return self._cs_request(url, 'POST', **kwargs)

    def put(self, url, **kwargs):
        return self._cs_request(url, 'PUT', **kwargs)

    def delete(self, url, **kwargs):
        return self._cs_request(url, 'DELETE', **kwargs)

    @classmethod
    def log_request(cls, url, method, data=None):
        if log.isEnabledFor(logging.DEBUG):
            log.debug('REQ URL: [{}] {}'.format(method, url))
            cls._debug_print_json(data, 'REQ BODY:')

    @staticmethod
    def _debug_print_json(data, prefix=None):
        if prefix is None:
            prefix = 'JSON data:'

        if data is not None and log.isEnabledFor(logging.DEBUG):
            try:
                if isinstance(data, six.string_types):
                    obj = json.loads(data, encoding='utf-8')
                else:
                    obj = data
                text = json.dumps(obj, indent=4)
            except (ValueError, TypeError):
                text = data
            log.debug('{}\n{}'.format(prefix, text))

    @classmethod
    def log_response(cls, full_url, method, resp, start_time):
        if log.isEnabledFor(logging.DEBUG):
            dt = time.time() - start_time
            log.debug('REQ URL: [{}] {}, TIME: {}, RESP CODE: {}'
                      .format(method, full_url, dt, resp.status_code))
            cls._debug_print_json(resp.text, 'RESP BODY:')

    def update_headers(self, headers):
        self.headers.update(headers)



# coding=utf-8
# Copyright (c) 2020 Dell Inc. or its subsidiaries.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from __future__ import unicode_literals

import persistqueue


class SGCache(persistqueue.PDict):
    """SGCache shares the VNX SG caching between OpenStack Cinder backends.

    This shared cache is introduced to fix the issue like:
    https://github.com/emc-openstack/storops/issues/305. The fix to storops is
    not enough though. Because different OpenStack VNX Cinder backends for
    two or more pools will have the dirty cache as described in issue #305.

    The shared cache will persist the VNX storage groups information in the
    same file path on the same machine via ``persistqueue.PDict``.
    """

    def __init__(self, persist_path):
        self._persist_path = persist_path
        super(SGCache, self).__init__(persist_path, 'storops_sg_cache',
                                      multithreading=True)



# coding=utf-8
# Copyright (c) 2017 Dell Inc. or its subsidiaries.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from __future__ import unicode_literals

import operator
from unittest import TestCase

import ddt
from hamcrest import assert_that, equal_to

from storops.lib.models import UnityModel


@ddt.ddt
class TestUnityModel(TestCase):
    def test_model_info(self):
        unity_model = UnityModel('Unity 880F')
        assert_that(unity_model.model, equal_to('880F'))
        assert_that(unity_model.model_base, equal_to('880'))
        assert_that(unity_model.series, equal_to('8'))
        assert_that(unity_model.is_all_flash, equal_to(True))

    @ddt.data(
        {'model': 'Unity 380F', 'expected': True},
        {'model': 'Unity 380', 'expected': False},
    )
    @ddt.unpack
    def test_is_all_flash(self, model, expected):
        unity_model = UnityModel(model)
        assert_that(unity_model.is_all_flash, equal_to(expected))

    @ddt.data(
        {'model_1': 'Unity 880F', 'model_2': 'Unity 870F', 'expected': True},
        {'model_1': 'Unity 880F', 'model_2': 'Unity 890F', 'expected': True},
        {'model_1': 'Unity 880F', 'model_2': 'Unity 880', 'expected': False},
        {'model_1': 'Unity 880', 'model_2': 'Unity 880F', 'expected': False},
        {'model_1': 'Unity 680F', 'model_2': 'Unity 880F', 'expected': False},
    )
    @ddt.unpack
    def test_is_same_model_series(self, model_1, model_2, expected):
        unity_model_1 = UnityModel(model_1)
        unity_model_2 = UnityModel(model_2)
        assert_that(unity_model_1.is_same_series(unity_model_2),
                    equal_to(expected))

    @ddt.data(
        {'model_1': 'Unity 880F', 'model_2': 'Unity 880F',
         'op': operator.eq, 'expected': True},
        {'model_1': 'Unity 880F', 'model_2': 'Unity 880',
         'op': operator.eq, 'expected': False},
        {'model_1': 'Unity 880', 'model_2': 'Unity 880F',
         'op': operator.eq, 'expected': False},
        {'model_1': 'Unity 880F', 'model_2': 'Unity 870F',
         'op': operator.eq, 'expected': False},
        {'model_1': 'Unity 380', 'model_2': 'Unity 380',
         'op': operator.eq, 'expected': True},
        {'model_1': 'Unity 880F', 'model_2': 'Unity 870F',
         'op': operator.gt, 'expected': True},
        {'model_1': 'Unity 880F', 'model_2': 'Unity 880F',
         'op': operator.gt, 'expected': False},
        {'model_1': 'Unity 880F', 'model_2': 'Unity 890F',
         'op': operator.gt, 'expected': False},
        {'model_1': 'Unity 880F', 'model_2': 'Unity 770F',
         'op': operator.gt, 'expected': False},
        {'model_1': 'Unity 880F', 'model_2': 'Unity 870',
         'op': operator.gt, 'expected': False},
        {'model_1': 'Unity 380', 'model_2': 'Unity 370',
         'op': operator.gt, 'expected': True},
        {'model_1': 'Unity 880F', 'model_2': 'Unity 870F',
         'op': operator.ge, 'expected': True},
        {'model_1': 'Unity 880F', 'model_2': 'Unity 880F',
         'op': operator.ge, 'expected': True},
        {'model_1': 'Unity 880F', 'model_2': 'Unity 890F',
         'op': operator.ge, 'expected': False},
        {'model_1': 'Unity 880F', 'model_2': 'Unity 770F',
         'op': operator.ge, 'expected': False},
        {'model_1': 'Unity 880F', 'model_2': 'Unity 870',
         'op': operator.ge, 'expected': False},
        {'model_1': 'Unity 380', 'model_2': 'Unity 370',
         'op': operator.ge, 'expected': True},
        {'model_1': 'Unity 880F', 'model_2': 'Unity 870F',
         'op': operator.lt, 'expected': False},
        {'model_1': 'Unity 880F', 'model_2': 'Unity 880F',
         'op': operator.lt, 'expected': False},
        {'model_1': 'Unity 880F', 'model_2': 'Unity 890F',
         'op': operator.lt, 'expected': True},
        {'model_1': 'Unity 880F', 'model_2': 'Unity 770F',
         'op': operator.lt, 'expected': False},
        {'model_1': 'Unity 880F', 'model_2': 'Unity 870',
         'op': operator.lt, 'expected': False},
        {'model_1': 'Unity 380', 'model_2': 'Unity 370',
         'op': operator.lt, 'expected': False},
        {'model_1': 'Unity 880F', 'model_2': 'Unity 870F',
         'op': operator.le, 'expected': False},
        {'model_1': 'Unity 880F', 'model_2': 'Unity 880F',
         'op': operator.le, 'expected': True},
        {'model_1': 'Unity 880F', 'model_2': 'Unity 890F',
         'op': operator.le, 'expected': True},
        {'model_1': 'Unity 880F', 'model_2': 'Unity 770F',
         'op': operator.le, 'expected': False},
        {'model_1': 'Unity 880F', 'model_2': 'Unity 870',
         'op': operator.le, 'expected': False},
        {'model_1': 'Unity 380', 'model_2': 'Unity 370',
         'op': operator.le, 'expected': False},
    )
    @ddt.unpack
    def test_model_compare(self, model_1, model_2, op, expected):
        unity_model_1 = UnityModel(model_1)
        unity_model_2 = UnityModel(model_2)
        assert_that(op(unity_model_1, unity_model_2), equal_to(expected))



# Copyright (c) 2015 EMC Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import unicode_literals

from unittest import TestCase

import ddt

from hamcrest import assert_that, equal_to, instance_of, raises

from storops.exception import UnityResourceNotFoundError
from storops.unity.resource.user_quota import UnityUserQuota, \
     UnityUserQuotaList
from storops_test.unity.rest_mock import t_rest, patch_rest

__author__ = 'Rajendra Indukuri'


@ddt.ddt
class UnityUserQuotaListTest(TestCase):
    @patch_rest
    def test_get_all(self):
        user_quotas = UnityUserQuotaList(cli=t_rest())
        assert_that(user_quotas, instance_of(UnityUserQuotaList))
        assert_that(len(user_quotas), equal_to(2))

    @patch_rest
    def test_get_specific_user_quota(self):
        user_quota = UnityUserQuota(
                      _id='userquota_171798692187_3_3', cli=t_rest())
        assert_that(user_quota, instance_of(UnityUserQuota))
        assert_that(user_quota.id, equal_to('userquota_171798692187_3_3'))
        assert_that(user_quota.existed, equal_to(True))

    @patch_rest
    def test_get_user_quota_not_found(self):
        user_quota = UnityUserQuota(
                      _id='abc', cli=t_rest())
        assert_that(user_quota.existed, equal_to(False))

    @patch_rest
    def test_create_user_quota(self):
        ret = UnityUserQuota.create(cli=t_rest(),
                                    filesystem_id='fs_2',
                                    hard_limit=9663676416,
                                    soft_limit=3221225472,
                                    uid=3)
        assert_that(ret, instance_of(UnityUserQuota))
        assert_that(ret.id, equal_to('userquota_171798692187_3_3'))
        assert_that(ret.existed, equal_to(True))

    @patch_rest
    def test_create_user_quota_invalid_filesystem_id(self):
        def f():
            UnityUserQuota.create(cli=t_rest(),
                                  filesystem_id='fs_99',
                                  hard_limit=9663676416,
                                  soft_limit=3221225472,
                                  uid=3)

        assert_that(f, raises(UnityResourceNotFoundError))

    @patch_rest
    def test_create_user_quota_on_tree_quota(self):
        ret = UnityUserQuota.create(cli=t_rest(), filesystem_id='fs_2',
                                    tree_quota_id='treequota_171798692187_3',
                                    hard_limit=9663676416,
                                    soft_limit=3221225472,
                                    uid=3)
        assert_that(ret, instance_of(UnityUserQuota))
        assert_that(ret.id, equal_to('userquota_171798692187_3_3'))
        assert_that(ret.existed, equal_to(True))

    @patch_rest
    def test_create_user_quota_on_tree_quota_fs_invalid(self):
        def f():
            UnityUserQuota.create(cli=t_rest(), filesystem_id='fs_99',
                                  tree_quota_id='treequota_171798692187_3',
                                  hard_limit=9663676416,
                                  soft_limit=3221225472,
                                  uid=3)

        assert_that(f, raises(UnityResourceNotFoundError))

    @patch_rest
    def test_create_user_quota_on_tree_quota_tq_invalid(self):
        def f():
            UnityUserQuota.create(cli=t_rest(), filesystem_id='fs_2',
                                  tree_quota_id='abc',
                                  hard_limit=9663676416,
                                  soft_limit=3221225472,
                                  uid=3)

        assert_that(f, raises(UnityResourceNotFoundError))

    @patch_rest
    def test_create_user_quota_on_tree_quota_tq_n_fs_invalid(self):
        def f():
            UnityUserQuota.create(cli=t_rest(), filesystem_id='fs_99',
                                  tree_quota_id='abc',
                                  hard_limit=9663676416,
                                  soft_limit=3221225472,
                                  uid=3)

        assert_that(f, raises(UnityResourceNotFoundError))

    @patch_rest
    def test_modify_user_quota(self):
        resp = UnityUserQuota.modify(
                cli=t_rest(), user_quota_id='userquota_171798692187_3_3',
                hard_limit=8589934592, soft_limit=2147483648)
        assert_that(resp.is_ok(), equal_to(True))

    @patch_rest
    def test_modify_user_quota_user_quota_id_invalid(self):
        def f():
            UnityUserQuota.modify(
                cli=t_rest(), user_quota_id='abc',
                hard_limit=8589934592, soft_limit=2147483648)

        assert_that(f, raises(UnityResourceNotFoundError))



# coding=utf-8
# Copyright (c) 2015 EMC Corporation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from __future__ import unicode_literals

import mock

__author__ = 'Jay Xu'

Ki = 1024

STORAGE_GROUP_HBA = """
HBA UID                                          SP Name  SPPort
-------                                          -------  ------
iqn.1991-05.com.microsoft:abc.def.dev             SP A     3
Host name:             abc.def.dev
SPPort:                A-3v1
Initiator IP:          10.244.209.72
TPGT:                  1
ISID:                  10000000000
"""

patch_retry = mock.patch(target='storops.lib.common.const_seconds',
                         new=lambda x: 0)
